@@ -30,6 +30,7 @@ The bootloader dialog is used to update the Crazyflie firmware and to
 read/write the configuration block in the Crazyflie flash.
 """
 from cflib.bootloader import Bootloader
+from cfclient.ui.connectivity_manager import ConnectivityManager
 
 import tempfile
 import logging
@@ -44,6 +45,7 @@ from PyQt5 import QtWidgets, uic
 from PyQt5.QtCore import pyqtSlot, pyqtSignal, QThread
 
 import cfclient
+import cflib.crazyflie
 
 __author__ = 'Bitcraze AB'
 __all__ = ['BootloaderDialog']
@@ -58,19 +60,20 @@ RELEASE_URL = 'https://api.github.com/repos/bitcraze/'\
               'crazyflie-release/releases'
 
 
-class UIState:
-    DISCONNECTED = 0
-    CONNECTING = 5
-    CONNECT_FAILED = 1
-    COLD_CONNECT = 2
-    FLASHING = 3
-    RESET = 4
-
-
 class BootloaderDialog(QtWidgets.QWidget, service_dialog_class):
     """Tab for update the Crazyflie firmware and for reading/writing the config
     block in flash"""
 
+    class UIState:
+        DISCONNECTED = 0
+        COLD_CONNECTING = 1
+        COLD_CONNECTED = 2
+        FW_CONNECTING = 3
+        FW_CONNECTED = 4
+        FW_SCANNING = 5
+        FLASHING = 6
+        RESET = 7
+
     _release_firmwares_found = pyqtSignal(object)
     _release_downloaded = pyqtSignal(str, object)
 
@@ -82,89 +85,138 @@ class BootloaderDialog(QtWidgets.QWidget, service_dialog_class):
         self.menuName = "Service"
 
         # self.tabWidget = tabWidget
-        self.helper = helper
+        self._helper = helper
 
         # self.cf = crazyflie
-        self.clt = CrazyloadThread()
+        self.clt = CrazyloadThread(self._helper.cf)
 
         # Connecting GUI signals (a pity to do that manually...)
         self.imagePathBrowseButton.clicked.connect(self.pathBrowse)
         self.programButton.clicked.connect(self.programAction)
         self.coldBootButton.clicked.connect(self.initiateColdboot)
         self.resetButton.clicked.connect(self.resetCopter)
-        self._cancel_bootloading.clicked.connect(self.close)
-        self.sourceTab.currentChanged.connect(
-            lambda _: self.updateChipSelectRadio())
+
+        self._helper.connectivity_manager.register_ui_elements(
+            ConnectivityManager.UiElementsContainer(
+                interface_combo=self.comboBox,
+                address_spinner=self.address,
+                connect_button=self.connectButton,
+                scan_button=self.scanButton))
+        self._helper.connectivity_manager.connection_state_changed.connect(self._fw_connection_state_changed)
 
         # connecting other signals
         self.clt.programmed.connect(self.programDone)
         self.clt.statusChanged.connect(self.statusUpdate)
         # self.clt.updateBootloaderStatusSignal.connect(
         #                                         self.updateBootloaderStatus)
-        self.clt.connectingSignal.connect(
-            lambda: self.setUiState(UIState.CONNECTING))
-        self.clt.connectedSignal.connect(
-            lambda: self.setUiState(UIState.COLD_CONNECT))
+        self.clt.connectingSignal.connect(lambda: self.setUiState(self.UIState.COLD_CONNECTING))
+
+        self.clt.connectedSignal.connect(lambda: self._load_thread_connection_event(self.UIState.COLD_CONNECTED))
+        self.clt.disconnectedSignal.connect(lambda: self._load_thread_connection_event(self.UIState.DISCONNECTED))
         self.clt.failed_signal.connect(lambda m: self._ui_connection_fail(m))
-        self.clt.disconnectedSignal.connect(
-            lambda: self.setUiState(UIState.DISCONNECTED))
+
+        self._cold_boot_error_message = None
+        self._state = self.UIState.DISCONNECTED
 
         self._releases = {}
         self._release_firmwares_found.connect(self._populate_firmware_dropdown)
         self._release_downloaded.connect(self.release_zip_downloaded)
-        self.firmware_downloader = FirmwareDownloader(
-                                        self._release_firmwares_found,
-                                        self._release_downloaded)
+        self.firmware_downloader = FirmwareDownloader(self._release_firmwares_found, self._release_downloaded)
         self.firmware_downloader.get_firmware_releases()
 
         self.firmware_downloader.start()
         self.clt.start()
 
     def _ui_connection_fail(self, message):
-        self.setStatusLabel(message)
-        self.coldBootButton.setEnabled(True)
+        self._cold_boot_error_message = message
+        self.setUiState(self.UIState.DISCONNECTED)
 
     def setUiState(self, state):
-        if (state == UIState.DISCONNECTED):
+        self._state = state
+
+        if (state == self.UIState.DISCONNECTED):
             self.resetButton.setEnabled(False)
             self.programButton.setEnabled(False)
-            self.setStatusLabel("Not connected")
+            if self._cold_boot_error_message is not None:
+                self.setStatusLabel(self._cold_boot_error_message)
+            else:
+                self.setStatusLabel("Not connected")
             self.coldBootButton.setEnabled(True)
             self.progressBar.setTextVisible(False)
             self.progressBar.setValue(0)
             self.statusLabel.setText('Status: <b>IDLE</b>')
-        elif (state == UIState.CONNECTING):
+            self.setSourceSelectionUiEnabled(True)
+            self._helper.connectivity_manager.set_enable(True)
+        elif (state == self.UIState.COLD_CONNECTING):
+            self._cold_boot_error_message = None
             self.resetButton.setEnabled(False)
             self.programButton.setEnabled(False)
-            self.setStatusLabel("Trying to connect cold bootloader, restart "
-                                "the Crazyflie to connect")
+            self.setStatusLabel("Trying to connect cold bootloader, restart the Crazyflie to connect")
             self.coldBootButton.setEnabled(False)
-        elif (state == UIState.CONNECT_FAILED):
-            self.setStatusLabel("Connecting to bootloader failed")
-            self.coldBootButton.setEnabled(True)
-        elif (state == UIState.COLD_CONNECT):
+            self.setSourceSelectionUiEnabled(True)
+            self._helper.connectivity_manager.set_enable(False)
+        elif (state == self.UIState.COLD_CONNECTED):
+            self._cold_boot_error_message = None
             self.resetButton.setEnabled(True)
             self.programButton.setEnabled(True)
             self.setStatusLabel("Connected to bootloader")
             self.coldBootButton.setEnabled(False)
-        elif (state == UIState.RESET):
+            self.imagePathBrowseButton.setEnabled(True)
+            self.imagePathLine.setEnabled(True)
+            self.firmwareDropdown.setEnabled(True)
+            self._helper.connectivity_manager.set_enable(False)
+        elif (state == self.UIState.FW_CONNECTING):
+            self._cold_boot_error_message = None
+            self.resetButton.setEnabled(False)
+            self.programButton.setEnabled(False)
+            self.setStatusLabel("Trying to connect in firmware mode")
+            self.coldBootButton.setEnabled(False)
+            self.setSourceSelectionUiEnabled(True)
+            self._helper.connectivity_manager.set_enable(True)
+        elif (state == self.UIState.FW_CONNECTED):
+            self._cold_boot_error_message = None
+            self.resetButton.setEnabled(False)
+            self.programButton.setEnabled(True)
+            self.setStatusLabel("Connected in firmware mode")
+            self.coldBootButton.setEnabled(False)
+            self.setSourceSelectionUiEnabled(True)
+            self._helper.connectivity_manager.set_enable(True)
+        elif (state == self.UIState.FW_SCANNING):
+            self._cold_boot_error_message = None
+            self.resetButton.setEnabled(False)
+            self.programButton.setEnabled(False)
+            self.setStatusLabel("Scanning")
+            self.coldBootButton.setEnabled(False)
+            self.setSourceSelectionUiEnabled(True)
+            self._helper.connectivity_manager.set_enable(True)
+        elif (state == self.UIState.FLASHING):
+            self.resetButton.setEnabled(False)
+            self.resetButton.setEnabled(False)
+            self.programButton.setEnabled(False)
+            self.setStatusLabel("Flashing")
+            self.coldBootButton.setEnabled(False)
+            self.setSourceSelectionUiEnabled(False)
+            self._helper.connectivity_manager.set_enable(False)
+        elif (state == self.UIState.RESET):
+            self._cold_boot_error_message = None
             self.setStatusLabel("Resetting to firmware, disconnected")
             self.resetButton.setEnabled(False)
             self.programButton.setEnabled(False)
             self.coldBootButton.setEnabled(False)
+            self.setSourceSelectionUiEnabled(True)
+            self._helper.connectivity_manager.set_enable(False)
+
+    def setSourceSelectionUiEnabled(self, enabled):
+        self.imagePathBrowseButton.setEnabled(enabled)
+        self.imagePathLine.setEnabled(enabled)
+        self.firmwareDropdown.setEnabled(enabled)
 
     def setStatusLabel(self, text):
         self.connectionStatus.setText("Status: <b>%s</b>" % text)
 
-    def connected(self):
-        self.setUiState(UIState.COLD_CONNECT)
-
-    def connectionFailed(self):
-        self.setUiState(UIState.CONNECT_FAILED)
-
     def resetCopter(self):
         self.clt.resetCopterSignal.emit()
-        self.setUiState(UIState.RESET)
+        self.setUiState(self.UIState.RESET)
 
     def updateConfig(self, channel, speed, rollTrim, pitchTrim):
         self.rollTrim.setValue(rollTrim)
@@ -176,8 +228,6 @@ class BootloaderDialog(QtWidgets.QWidget, service_dialog_class):
         self.clt.terminate_flashing()
         # Remove downloaded-firmware files.
         self.firmware_downloader.bootload_complete.emit()
-        self.setUiState(UIState.RESET)
-        self.clt.resetCopterSignal.emit()
 
     def _populate_firmware_dropdown(self, releases):
         """ Callback from firmware-downloader that retrieves all
@@ -198,39 +248,44 @@ class BootloaderDialog(QtWidgets.QWidget, service_dialog_class):
         self.downloadStatus.setText('Downloaded')
         self.clt.program.emit(release_path, '')
 
-    def updateChipSelectRadio(self):
-        if self.sourceTab.currentWidget() == self.tabFromFile:
-            if self.imagePathLine.text().endswith(".zip"):
-                self.radioBoth.setEnabled(True)
-                self.radioBoth.setChecked(True)
-            elif self.imagePathLine.text().endswith(".bin"):
-                self.radioBoth.setEnabled(False)
-                self.radioStm32.setChecked(True)
-        else:
-            self.radioBoth.setEnabled(True)
-            self.radioBoth.setChecked(True)
+    def _load_thread_connection_event(self, new_sate):
+        if self._state != self.UIState.FLASHING:
+            self.setUiState(new_sate)
+
+    def _fw_connection_state_changed(self, new_state):
+        if self._state != self.UIState.FLASHING:
+            if new_state == ConnectivityManager.UIState.DISCONNECTED:
+                self.setUiState(self.UIState.DISCONNECTED)
+            elif new_state == ConnectivityManager.UIState.CONNECTING:
+                self.setUiState(self.UIState.FW_CONNECTING)
+            elif new_state == ConnectivityManager.UIState.CONNECTED:
+                self.setUiState(self.UIState.FW_CONNECTED)
+            elif new_state == ConnectivityManager.UIState.SCANNING:
+                self.setUiState(self.UIState.FW_SCANNING)
 
     @pyqtSlot()
     def pathBrowse(self):
-        filename = ""
-        # Fix for crash in X on Ubuntu 14.04
-        filename, _ = QtWidgets.QFileDialog.getOpenFileName()
-        if filename != "" and filename[-4:] in (".bin", ".zip"):
-            self.imagePathLine.setText(filename)
-            self.updateChipSelectRadio()
-        elif filename != "":
-            msgBox = QtWidgets.QMessageBox()
-            msgBox.setText("Wrong file extention. Must be .bin or .zip.")
-            msgBox.exec_()
+        names = QtWidgets.QFileDialog.getOpenFileName(
+            self, 'Release file to flash', self._helper.current_folder, "*.zip")
+        if names[0] == '':
+            return
 
-        pass
+        filename = names[0]
+        self._helper.current_folder = os.path.dirname(filename)
+
+        if filename.endswith('.zip'):
+            self.imagePathLine.setText(filename)
+        else:
+            msgBox = QtWidgets.QMessageBox()
+            msgBox.setText("Wrong file extention. Must be .zip.")
+            msgBox.exec_()
 
     @pyqtSlot()
     def programAction(self):
-        # self.setStatusLabel("Initiate programming")
-        self.resetButton.setEnabled(False)
-        self.programButton.setEnabled(False)
-        self.imagePathBrowseButton.setEnabled(False)
+        if self._state == self.UIState.COLD_CONNECTED:
+            self.clt.set_boot_mode(self.clt.COLD_BOOT)
+        else:
+            self.clt.set_boot_mode(self.clt.WARM_BOOT)
 
         # call the flasher
         if self.sourceTab.currentWidget() == self.tabFromFile:
@@ -239,38 +294,31 @@ class BootloaderDialog(QtWidgets.QWidget, service_dialog_class):
                 msgBox.setText("Please choose an image file to program.")
                 msgBox.exec_()
 
-                self.resetButton.setEnabled(True)
-                self.programButton.setEnabled(True)
-                self.imagePathBrowseButton.setEnabled(True)
                 return
 
-            # by default, both of the mcu:s are flashed
-            mcu_to_flash = None
+            self.setUiState(self.UIState.FLASHING)
 
-            if self.radioStm32.isChecked():
-                mcu_to_flash = 'stm32'
-            elif self.radioNrf51.isChecked():
-                mcu_to_flash = 'nrf51'
+            # by default, flash everything in the zip (if possible)
+            mcu_to_flash = None
             self.clt.program.emit(self.imagePathLine.text(), mcu_to_flash)
         else:
+            self.setUiState(self.UIState.FLASHING)
+
             requested_release = self.firmwareDropdown.currentText()
             download_url = self._releases[requested_release]
             self.downloadStatus.setText('Fetching...')
-            self.firmware_downloader.download_release(requested_release,
-                                                      download_url)
+            self.firmware_downloader.download_release(requested_release, download_url)
 
     @pyqtSlot(bool)
     def programDone(self, success):
         if success:
             self.statusLabel.setText('Status: <b>Programing complete!</b>')
             self.downloadStatus.setText('')
-
         else:
             self.statusLabel.setText('Status: <b>Programing failed!</b>')
 
-        self.resetButton.setEnabled(True)
-        self.programButton.setEnabled(True)
-        self.imagePathBrowseButton.setEnabled(True)
+        self.setUiState(self.UIState.DISCONNECTED)
+        self.resetCopter()
 
     @pyqtSlot(str, int)
     def statusUpdate(self, status, progress):
@@ -304,7 +352,10 @@ class CrazyloadThread(QThread):
 
     radioSpeedPos = 2
 
-    def __init__(self):
+    WARM_BOOT = 0
+    COLD_BOOT = 1
+
+    def __init__(self, crazyflie: cflib.crazyflie.Crazyflie):
         super(CrazyloadThread, self).__init__()
 
         self._terminate_flashing = False
@@ -313,6 +364,10 @@ class CrazyloadThread(QThread):
         self._bl.progress_cb = self.statusChanged.emit
         self._bl.terminate_flashing_cb = lambda: self._terminate_flashing
 
+        self._cf = crazyflie
+
+        self._boot_mode = self.COLD_BOOT
+
         # Make sure that the signals are handled by this thread event loop
         self.moveToThread(self)
 
@@ -324,6 +379,9 @@ class CrazyloadThread(QThread):
         self.quit()
         self.wait()
 
+    def set_boot_mode(self, mode):
+        self._boot_mode = mode
+
     def initiateColdBoot(self, linkURI):
         self.connectingSignal.emit()
 
@@ -336,7 +394,24 @@ class CrazyloadThread(QThread):
         except Exception as e:
             self.failed_signal.emit("{}".format(e))
 
+    def rebootToBootloader(self):
+        self._bl.clink = self._cf.link_uri
+        self._cf.close_link()
+
+        try:
+            success = self._bl.start_bootloader(warm_boot=True)
+            if not success:
+                self.failed_signal.emit("Could not connect to bootloader")
+            else:
+                self.connectedSignal.emit()
+        except Exception as e:
+            self.failed_signal.emit("{}".format(e))
+
     def programAction(self, filename, mcu_to_flash):
+
+        if self._boot_mode == self.WARM_BOOT:
+            self.rebootToBootloader()
+
         targets = {}
         if mcu_to_flash:
             targets[mcu_to_flash] = ("fw",)
