@@ -7,7 +7,7 @@
 #  +------+    / /_/ / / /_/ /__/ /  / /_/ / / /_/  __/
 #   ||  ||    /_____/_/\__/\___/_/   \__,_/ /___/\___/
 #
-#  Copyright (C) 2011-2013 Bitcraze AB
+#  Copyright (C) 2011-2021 Bitcraze AB
 #
 #  Crazyflie Nano Quadcopter Client
 #
@@ -30,6 +30,8 @@ The flight control tab shows telemetry data and flight settings.
 """
 
 import logging
+import time
+from enum import Enum
 
 from PyQt5 import uic
 from PyQt5.QtCore import Qt, pyqtSignal
@@ -40,6 +42,7 @@ from cfclient.ui.widgets.ai import AttitudeIndicator
 
 from cfclient.utils.config import Config
 from cflib.crazyflie.log import LogConfig
+from cflib.positioning.position_hl_commander import PositionHlCommander
 
 from cfclient.utils.input import JoystickReader
 
@@ -79,12 +82,22 @@ keep the position in X and Y as well. Thrust control becomes height velocity
 control. Requires a flow deck. Uses body-fixed coordinates."""
 
 
+class CommanderAction(Enum):
+    TAKE_OFF = 1
+    LAND = 2
+    UP = 3
+    DOWN = 4
+    LEFT = 5
+    RIGHT = 6
+    FORWARD = 7
+    BACK = 8
+
+
 class FlightTab(Tab, flight_tab_class):
     uiSetupReadySignal = pyqtSignal()
 
-    _motor_data_signal = pyqtSignal(int, object, object)
-    _imu_data_signal = pyqtSignal(int, object, object)
-    _baro_data_signal = pyqtSignal(int, object, object)
+    _log_data_signal = pyqtSignal(int, object, object)
+    _pose_data_signal = pyqtSignal(object, object)
 
     _input_updated_signal = pyqtSignal(float, float, float, float)
     _rp_trim_updated_signal = pyqtSignal(float, float)
@@ -144,9 +157,8 @@ class FlightTab(Tab, flight_tab_class):
         self._assisted_control_updated_signal.connect(
             self._assisted_control_updated)
 
-        self._imu_data_signal.connect(self._imu_data_received)
-        self._baro_data_signal.connect(self._baro_data_received)
-        self._motor_data_signal.connect(self._motor_data_received)
+        self._pose_data_signal.connect(self._pose_data_received)
+        self._log_data_signal.connect(self._log_data_received)
 
         self._log_error_signal.connect(self._logging_error)
 
@@ -164,6 +176,35 @@ class FlightTab(Tab, flight_tab_class):
         self.maxYawRate.valueChanged.connect(self.maxYawRateChanged)
         self.uiSetupReadySignal.connect(self.uiSetupReady)
         self.isInCrazyFlightmode = False
+
+        # Command Based Flight Control
+        self._can_fly = 0
+        self._hlCommander = None
+        self.commanderTakeOffButton.clicked.connect(
+            lambda: self._flight_command(CommanderAction.TAKE_OFF)
+        )
+        self.commanderLandButton.clicked.connect(
+            lambda: self._flight_command(CommanderAction.LAND)
+        )
+        self.commanderLeftButton.clicked.connect(
+            lambda: self._flight_command(CommanderAction.LEFT)
+        )
+        self.commanderRightButton.clicked.connect(
+            lambda: self._flight_command(CommanderAction.RIGHT)
+        )
+        self.commanderForwardButton.clicked.connect(
+            lambda: self._flight_command(CommanderAction.FORWARD)
+        )
+        self.commanderBackButton.clicked.connect(
+            lambda: self._flight_command(CommanderAction.BACK)
+        )
+        self.commanderUpButton.clicked.connect(
+            lambda: self._flight_command(CommanderAction.UP)
+        )
+        self.commanderDownButton.clicked.connect(
+            lambda: self._flight_command(CommanderAction.DOWN)
+        )
+
         self.uiSetupReady()
 
         self.ratePidRadioButton.clicked.connect(
@@ -205,7 +246,6 @@ class FlightTab(Tab, flight_tab_class):
         self.helper.cf.param.all_updated.add_callback(
             self._all_params_updated)
 
-        self.logBaro = None
         self.logAltHold = None
 
         self.ai = AttitudeIndicator()
@@ -227,6 +267,9 @@ class FlightTab(Tab, flight_tab_class):
             self._limiting_updated.emit)
         self._limiting_updated.connect(self._set_limiting_enabled)
 
+        self.helper.pose_logger.data_received_cb.add_callback(
+            self._pose_data_signal.emit)
+
     def _set_limiting_enabled(self, rp_limiting_enabled,
                               yaw_limiting_enabled,
                               thrust_limiting_enabled):
@@ -252,27 +295,67 @@ class FlightTab(Tab, flight_tab_class):
             self.flightModeCombo.setCurrentIndex(flightComboIndex)
             self.flightModeCombo.currentIndexChanged.emit(flightComboIndex)
 
+    def _flight_command(self, action):
+        if self._hlCommander is None:
+            return
+
+        if action == CommanderAction.TAKE_OFF:
+            #
+            # Reset the Kalman filter before taking off, to avoid
+            # positional confusion.
+            #
+            self.helper.cf.param.set_value('kalman.resetEstimation', '1')
+            time.sleep(1)
+            self._hlCommander.take_off()
+        elif action == CommanderAction.LAND:
+            self._hlCommander.land()
+        elif action == CommanderAction.LEFT:
+            self._hlCommander.left(0.5)
+        elif action == CommanderAction.RIGHT:
+            self._hlCommander.right(0.5)
+        elif action == CommanderAction.FORWARD:
+            self._hlCommander.forward(0.5)
+        elif action == CommanderAction.BACK:
+            self._hlCommander.back(0.5)
+        elif action == CommanderAction.UP:
+            self._hlCommander.up(0.5)
+        elif action == CommanderAction.DOWN:
+            self._hlCommander.down(0.5)
+
     def _logging_error(self, log_conf, msg):
         QMessageBox.about(self, "Log error",
                           "Error when starting log config [%s]: %s" % (
                               log_conf.name, msg))
 
-    def _motor_data_received(self, timestamp, data, logconf):
+    def _log_data_received(self, timestamp, data, logconf):
         if self.isVisible():
             self.actualM1.setValue(data["motor.m1"])
             self.actualM2.setValue(data["motor.m2"])
             self.actualM3.setValue(data["motor.m3"])
             self.actualM4.setValue(data["motor.m4"])
 
-    def _baro_data_received(self, timestamp, data, logconf):
+            self.estimateThrust.setText(
+                "%.2f%%" % self.thrustToPercentage(data["stabilizer.thrust"]))
+
+            if data["sys.canfly"] != self._can_fly:
+                self._can_fly = data["sys.canfly"]
+                self._update_flight_commander(True)
+
+    def _pose_data_received(self, pose_logger, pose):
         if self.isVisible():
-            estimated_x = data[LOG_NAME_ESTIMATE_X]
-            estimated_y = data[LOG_NAME_ESTIMATE_Y]
-            estimated_z = data[LOG_NAME_ESTIMATE_Z]
-            self.estimateX.setText(("%.2f" % estimated_x))
-            self.estimateY.setText(("%.2f" % estimated_y))
+            estimated_z = pose[2]
+            roll = pose[3]
+            pitch = pose[4]
+
+            self.estimateX.setText(("%.2f" % pose[0]))
+            self.estimateY.setText(("%.2f" % pose[1]))
             self.estimateZ.setText(("%.2f" % estimated_z))
+            self.estimateRoll.setText(("%.2f" % roll))
+            self.estimatePitch.setText(("%.2f" % pitch))
+            self.estimateYaw.setText(("%.2f" % pose[5]))
+
             self.ai.setBaro(estimated_z, self.is_visible())
+            self.ai.setRollPitch(-roll, pitch, self.is_visible())
 
     def _heighthold_input_updated(self, roll, pitch, yaw, height):
         if (self.isVisible() and
@@ -300,18 +383,6 @@ class FlightTab(Tab, flight_tab_class):
 
             self._change_input_labels(using_hover_assist=True)
 
-    def _imu_data_received(self, timestamp, data, logconf):
-        if self.isVisible():
-            self.estimateRoll.setText(("%.2f" % data["stabilizer.roll"]))
-            self.estimatePitch.setText(("%.2f" % data["stabilizer.pitch"]))
-            self.estimateYaw.setText(("%.2f" % data["stabilizer.yaw"]))
-            self.estimateThrust.setText("%.2f%%" %
-                                        self.thrustToPercentage(
-                                            data["stabilizer.thrust"]))
-
-            self.ai.setRollPitch(-data["stabilizer.roll"],
-                                 data["stabilizer.pitch"], self.is_visible())
-
     def _change_input_labels(self, using_hover_assist):
         if using_hover_assist:
             pitch, roll, yaw = 'Velocity X', 'Velocity Y', 'Velocity Z'
@@ -322,34 +393,57 @@ class FlightTab(Tab, flight_tab_class):
         self.inputRollLabel.setText(roll)
         self.inputYawLabel.setText(yaw)
 
+    def _update_flight_commander(self, connected):
+        self.commanderBox.setToolTip(str())
+        if not connected:
+            self.commanderBox.setEnabled(False)
+            return
+
+        if self._can_fly == 0:
+            self.commanderBox.setEnabled(False)
+            self.commanderBox.setToolTip(
+                'The Crazyflie reports that flight is not possible'
+            )
+            return
+
+        # We cannot know if we have a positioning deck until we get params
+        if not self.helper.cf.param.is_updated:
+            self.commanderBox.setEnabled(False)
+            return
+
+        #                  flowV1    flowV2     LightHouse       LPS
+        position_decks = ['bcFlow', 'bcFlow2', 'bcLighthouse4', 'bcDWM1000']
+        for deck in position_decks:
+            if int(self.helper.cf.param.values['deck'][deck]) == 1:
+                self.commanderBox.setEnabled(True)
+                break
+        else:
+            self.commanderBox.setToolTip(
+                'You need a positioning deck to use Command Based Flight'
+            )
+            self.commanderBox.setEnabled(False)
+
     def connected(self, linkURI):
-        # IMU & THRUST
-        lg = LogConfig("Stabilizer", Config().get("ui_update_period"))
-        lg.add_variable("stabilizer.roll", "float")
-        lg.add_variable("stabilizer.pitch", "float")
-        lg.add_variable("stabilizer.yaw", "float")
-        lg.add_variable("stabilizer.thrust", "uint16_t")
-
-        try:
-            self.helper.cf.log.add_config(lg)
-            lg.data_received_cb.add_callback(self._imu_data_signal.emit)
-            lg.error_cb.add_callback(self._log_error_signal.emit)
-            lg.start()
-        except KeyError as e:
-            logger.warning(str(e))
-        except AttributeError as e:
-            logger.warning(str(e))
-
-        # MOTOR
+        # MOTOR & THRUST
         lg = LogConfig("Motors", Config().get("ui_update_period"))
+        lg.add_variable("stabilizer.thrust", "uint16_t")
         lg.add_variable("motor.m1")
         lg.add_variable("motor.m2")
         lg.add_variable("motor.m3")
         lg.add_variable("motor.m4")
+        lg.add_variable("sys.canfly")
+
+        self._hlCommander = PositionHlCommander(
+            self.helper.cf,
+            x=0.0, y=0.0, z=0.0,
+            default_velocity=0.3,
+            default_height=0.5,
+            controller=PositionHlCommander.CONTROLLER_PID
+        )
 
         try:
             self.helper.cf.log.add_config(lg)
-            lg.data_received_cb.add_callback(self._motor_data_signal.emit)
+            lg.data_received_cb.add_callback(self._log_data_signal.emit)
             lg.error_cb.add_callback(self._log_error_signal.emit)
             lg.start()
         except KeyError as e:
@@ -367,26 +461,7 @@ class FlightTab(Tab, flight_tab_class):
         available = eval(available)
 
         self._enable_estimators(True)
-
         self.helper.inputDeviceReader.set_alt_hold_available(available)
-        if not self.logBaro:
-            # The sensor is available, set up the logging
-            self.logBaro = LogConfig("Baro", 200)
-            self.logBaro.add_variable(LOG_NAME_ESTIMATE_X, "float")
-            self.logBaro.add_variable(LOG_NAME_ESTIMATE_Y, "float")
-            self.logBaro.add_variable(LOG_NAME_ESTIMATE_Z, "float")
-
-            try:
-                self.helper.cf.log.add_config(self.logBaro)
-                self.logBaro.data_received_cb.add_callback(
-                    self._baro_data_signal.emit)
-                self.logBaro.error_cb.add_callback(
-                    self._log_error_signal.emit)
-                self.logBaro.start()
-            except KeyError as e:
-                logger.warning(str(e))
-            except AttributeError as e:
-                logger.warning(str(e))
 
     def disconnected(self, linkURI):
         self.ai.setRollPitch(0, 0)
@@ -409,7 +484,6 @@ class FlightTab(Tab, flight_tab_class):
 
         self._enable_estimators(False)
 
-        self.logBaro = None
         self.logAltHold = None
         self._led_ring_effect.setEnabled(False)
         self._led_ring_effect.clear()
@@ -431,6 +505,8 @@ class FlightTab(Tab, flight_tab_class):
         self._assist_mode_combo.setEnabled(False)
         self._assist_mode_combo.clear()
 
+        self._update_flight_commander(False)
+
     def minMaxThrustChanged(self):
         self.helper.inputDeviceReader.min_thrust = self.minThrust.value()
         self.helper.inputDeviceReader.max_thrust = self.maxThrust.value()
@@ -582,6 +658,7 @@ class FlightTab(Tab, flight_tab_class):
     def _all_params_updated(self):
         self._ring_populate_dropdown()
         self._populate_assisted_mode_dropdown()
+        self._update_flight_commander(True)
 
     def _ring_populate_dropdown(self):
         try:
